@@ -19,13 +19,25 @@ import pytest
 from sigstore._utils import read_embedded
 
 
-@pytest.mark.parametrize("env", ["prod", "staging"])
+@pytest.mark.parametrize(
+    "env",
+    [
+        "https://tuf-repo-cdn.sigstore.dev",
+        "https://tuf-repo-cdn.sigstage.dev",
+    ],
+)
 def test_store_reads_root_json(env):
     root_json = read_embedded("root.json", env)
     assert json.loads(root_json)
 
 
-@pytest.mark.parametrize("env", ["prod", "staging"])
+@pytest.mark.parametrize(
+    "env",
+    [
+        "https://tuf-repo-cdn.sigstore.dev",
+        "https://tuf-repo-cdn.sigstage.dev",
+    ],
+)
 def test_store_reads_targets_json(env):
     trusted_root_json = read_embedded("trusted_root.json", env)
     assert json.loads(trusted_root_json)
